#!/usr/bin/env ruby
#check_conf_cisco v0.1
#!/usr/bin/env ruby
#	
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#	Copyright Alain Pernelle
# Project:  www.alpern.free.fr
# Website: http://www.alpern.free.fr
# Quick documentation: use -h option
# TODO
# - keep name of save file with date.
#Utilisation:
#./check_cisco_config_v0.2.rb -H <ip_address> -u <username> -p <password> -f <enablepassword> -o <os>
# os can be ios, catos, h3c, enterasys
# -h, --help    Print detailed help screen
#version 0.2 08/14/2009 add support of catos h3c enterasys network devices 
# need device was setting with username and password
# Bug with Cisco Device Special password charatere

require 'date'
require 'thread'
require 'rubygems'
require 'optiflag'
require 'net/telnet'
now = DateTime.now
mois = now.month
annee = now.year
jour= now.day

# Gestion des options de ligne de commande
# puts "cisco_check_config_v0.1 by alpern  www.alpern.free.fr"
module Example  extend OptiFlagSet
  usage_flag "h","help","?"
  flag "H" do
    long_form "Host"
    description "Host, Cible Equipement Cisco "
  end
  flag "u" do
    long_form "username"
    description "username "
  end
  flag "p" do
    long_form "password"
    description "password "
  end
  flag "f" do
    long_form "enable"
    description "enable password "
  end
  optional_flag "c" do
    long_form "critical"
    description "--critical, default 60,  Critical time (s)"
  end
  flag "o" do
    long_form "os"
    description "--Operating System,values, ios,catos, h3c "
    value_in_set ["ios","catos","h3c","enterasys"]
  end 
  optional_flag "w" do
    long_form "warn"
    description "--warn, default 5,  warn time (s)"
  end
  optional_switch_flag "e" do
    description "extended mode, see the documentation"
  end 
  and_process!
end 

# GET THE ARGV VALUES
if ARGV.flags.e?
  EXTENDED=1
else
  EXTENDED=0
end
# Get Require Arguments 
  username=ARGV.flags.u
  password=ARGV.flags.p
  enablepassword=ARGV.flags.f
  peer=ARGV.flags.H
  os=ARGV.flags.o
# Get Optional arguments
if ARGV.flags.c?
  timeCritical=ARGV.flags.c.to_f
else
  timeCritical=60
end

if ARGV.flags.w?
  timeWarn=ARGV.flags.w.to_f
else
  timeWarn=5
end
begin

fichier=(peer)

case os
   when "catos"
       login_prompt = /[: ]*\z/n
       password_prompt = /[: ]*\z/n
       para = {
       "Name"          => username,
       "Password"  => password,
       "LoginPrompt"         => login_prompt,
       "PasswordPrompt"           => password_prompt} 
       localhost = Net::Telnet::new("Host" => peer,
                               "Timeout" => 25,
                               "Prompt" => /> /n
                             )
       localhost.login(para)
    when "ios"
       login_prompt = /[: ]*\z/n
       password_prompt = /[: ]*\z/n
       para = {
       "Name"          => username,
       "Password"  => password,
       "LoginPrompt"         => login_prompt,
       "PasswordPrompt"           => password_prompt} 
       localhost = Net::Telnet::new("Host" => peer,
                               "Timeout" => 25,
                               "Prompt" => /[#>]/n
                             )
       localhost.login(para)
    when "h3c"
        login_prompt = /Username:/
        password_prompt = /Password:/
        para = {
        "Name"                => username,
        "Password"            => password,
        "LoginPrompt"         => login_prompt,
        "PasswordPrompt"      => password_prompt }
        localhost = Net::Telnet::new("Host" => peer,
                               "Timeout" => 5,
                               "Prompt" => /[>]/,
                               "Waittime" => 0.01 )
        localhost.login(para)
    when "enterasys"
        login_prompt = /Username:/
        password_prompt = /Password:/
        para = {
        "Name"                => username,
        "Password"            => password,
        "LoginPrompt"         => login_prompt,
        "PasswordPrompt"      => password_prompt }
        localhost = Net::Telnet::new("Host" => peer,
                               "Timeout" => 5,
                               "Prompt" => /[>]/,
                               "Waittime" => 0.01 )
        localhost.login(para)   

   else
end
#  localhost.login(username,password) { |c| print c }
# Si le switches ou le router propose directement de taper le password
#  if username == password
#      localhost.waitfor(/[U]nk/n)
#          else
#  end

case os
  when "catos"
     localhost.puts("enable")
     localhost.cmd(enablepassword)
     localhost.cmd("set length 0")
     line = localhost.cmd("sh conf")
     localhost.cmd("exit")
     localhost.close  
  when "h3c"
     localhost.puts("super")
     localhost.cmd(enablepassword)
     localhost.cmd("screen-length disable")
     line = localhost.cmd("dis curr")
     localhost.cmd("quit")
     localhost.close
  when "enterasys"
#     localhost.puts("super")
#     localhost.cmd(enablepassword)
     localhost.cmd("set length 0")
     line = localhost.cmd("sh config")
     localhost.cmd("quit")
     localhost.close
   when "ios"
     localhost.puts("enable")
     localhost.cmd(enablepassword)
     localhost.cmd("term leng 0")
     line = localhost.cmd("sh running")
     localhost.cmd("exit")
     localhost.close
else
end
# Code de retour vers Nagios voir si line est vide
if line
  totalSize=line.length
  SortieFile = File.new("/var/log/nagios/"+fichier +"_"+jour.to_s+"-"+mois.to_s+"-"+annee.to_s+".confg", "w")
  SortieFile.puts(line)
  SortieFile.close
  retCode=0
  retCodeLabel="OK"
  puts "#{retCodeLabel}"+"|"+"size="+"#{totalSize}"+"B"
  exit retCode
else
  retCode=2
  retCodeLabel="Critical"
  puts "#{retCodeLabel}"
  exit retCode
end
rescue SystemCallError
  retCode=2
  retCodeLabel="Critical"
  print "#{retCodeLabel}"
  print "Erreur dans le programe"
  exit retCode
## print the script result for nagios
###############################################################
print "#{retCodeLabel}"
exit retCode
end 
